@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2004  The DOSBox Team
+ *  Copyright (C) 2002-2007  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,16 +15,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
 	CASE_0F_D(0x00)												/* GRP 6 Exxx */
 		{
+			if ((reg_flags & FLAG_VM) || (!cpu.pmode)) goto illegal_opcode;
 			GetRM;Bitu which=(rm>>3)&7;
 			switch (which) {
 			case 0x00:	/* SLDT */
 			case 0x01:	/* STR */
 				{
 					Bitu saveval;
-					if (!which) CPU_SLDT(saveval);
-					else CPU_STR(saveval);
+					if (!which) saveval=CPU_SLDT();
+					else saveval=CPU_STR();
 					if (rm >= 0xc0) {GetEArw;*earw=(Bit16u)saveval;}
 					else {GetEAa;SaveMw(eaa,saveval);}
 				}
@@ -32,20 +34,30 @@
 			case 0x02:case 0x03:case 0x04:case 0x05:
 				{
 					/* Just use 16-bit loads since were only using selectors */
-					FillFlags();
 					Bitu loadval;
 					if (rm >= 0xc0 ) {GetEArw;loadval=*earw;}
 					else {GetEAa;loadval=LoadMw(eaa);}
 					switch (which) {
-					case 0x02:CPU_LLDT(loadval);break;
-					case 0x03:CPU_LTR(loadval);break;
-					case 0x04:CPU_VERR(loadval);break;
-					case 0x05:CPU_VERW(loadval);break;
+					case 0x02:
+						if (cpu.cpl) EXCEPTION(EXCEPTION_GP);
+						if (CPU_LLDT(loadval)) RUNEXCEPTION();
+						break;
+					case 0x03:
+						if (cpu.cpl) EXCEPTION(EXCEPTION_GP);
+						if (CPU_LTR(loadval)) RUNEXCEPTION();
+						break;
+					case 0x04:
+						CPU_VERR(loadval);
+						break;
+					case 0x05:
+						CPU_VERW(loadval);
+						break;
 					}
 				}
 				break;
 			default:
 				LOG(LOG_CPU,LOG_ERROR)("GRP6:Illegal call %2X",which);
+				goto illegal_opcode;
 			}
 		}
 		break;
@@ -53,45 +65,54 @@
 		{
 			GetRM;Bitu which=(rm>>3)&7;
 			if (rm < 0xc0)	{ //First ones all use EA
-				GetEAa;Bitu limit,base;
+				GetEAa;Bitu limit;
 				switch (which) {
 				case 0x00:										/* SGDT */
-					CPU_SGDT(limit,base);
-					SaveMw(eaa,(Bit16u)limit);
-					SaveMd(eaa+2,(Bit32u)base);
+					SaveMw(eaa,(Bit16u)CPU_SGDT_limit());
+					SaveMd(eaa+2,(Bit32u)CPU_SGDT_base());
 					break;
 				case 0x01:										/* SIDT */
-					CPU_SIDT(limit,base);
-					SaveMw(eaa,(Bit16u)limit);
-					SaveMd(eaa+2,(Bit32u)base);
+					SaveMw(eaa,(Bit16u)CPU_SIDT_limit());
+					SaveMd(eaa+2,(Bit32u)CPU_SIDT_base());
 					break;
 				case 0x02:										/* LGDT */
+					if (cpu.pmode && cpu.cpl) EXCEPTION(EXCEPTION_GP);
 					CPU_LGDT(LoadMw(eaa),LoadMd(eaa+2));
 					break;
 				case 0x03:										/* LIDT */
+					if (cpu.pmode && cpu.cpl) EXCEPTION(EXCEPTION_GP);
 					CPU_LIDT(LoadMw(eaa),LoadMd(eaa+2));
 					break;
 				case 0x04:										/* SMSW */
-					CPU_SMSW(limit);
-					SaveMw(eaa,(Bit16u)limit);
+					SaveMw(eaa,(Bit16u)CPU_SMSW());
 					break;
 				case 0x06:										/* LMSW */
 					limit=LoadMw(eaa);
-					if (!CPU_LMSW((Bit16u)limit)) goto decode_end;
+					if (CPU_LMSW((Bit16u)limit)) RUNEXCEPTION();
+					break;
+				case 0x07:										/* INVLPG */
+					if (cpu.pmode && cpu.cpl) EXCEPTION(EXCEPTION_GP);
+					PAGING_ClearTLB();
 					break;
 				}
 			} else {
-				GetEArd;Bitu limit;
+				GetEArd;
 				switch (which) {
+				case 0x02:										/* LGDT */
+					if (cpu.pmode && cpu.cpl) EXCEPTION(EXCEPTION_GP);
+					goto illegal_opcode;
+				case 0x03:										/* LIDT */
+					if (cpu.pmode && cpu.cpl) EXCEPTION(EXCEPTION_GP);
+					goto illegal_opcode;
 				case 0x04:										/* SMSW */
-					CPU_SMSW(limit);
-					*eard=(Bit32u)limit;
+					*eard=(Bit32u)CPU_SMSW();
 					break;
 				case 0x06:										/* LMSW */
-					if (!CPU_LMSW(*eard)) goto decode_end;
+					if (CPU_LMSW(*eard)) RUNEXCEPTION();
 					break;
 				default:
 					LOG(LOG_CPU,LOG_ERROR)("Illegal group 7 RM subfunction %d",which);
+					goto illegal_opcode;
 					break;
 				}
 
@@ -100,7 +121,7 @@
 		break;
 	CASE_0F_D(0x02)												/* LAR Gd,Ed */
 		{
-			FillFlags();
+			if ((reg_flags & FLAG_VM) || (!cpu.pmode)) goto illegal_opcode;
 			GetRMrd;Bitu ar=*rmrd;
 			if (rm >= 0xc0) {
 				GetEArw;CPU_LAR(*earw,ar);
@@ -112,7 +133,7 @@
 		break;
 	CASE_0F_D(0x03)												/* LSL Gd,Ew */
 		{
-			FillFlags();
+			if ((reg_flags & FLAG_VM) || (!cpu.pmode)) goto illegal_opcode;
 			GetRMrd;Bitu limit=*rmrd;
 			/* Just load 16-bit values for selectors */
 			if (rm >= 0xc0) {
@@ -376,3 +397,10 @@
 			else {GetEAa;*rmrd=LoadMws(eaa);}
 			break;
 		}
+	CASE_0F_D(0xc1)												/* XADD Gd,Ed */
+		{
+			GetRMrd;Bit32u oldrmrd=*rmrd;
+			if (rm >= 0xc0 ) {GetEArd;*rmrd=*eard;*eard+=oldrmrd;}
+			else {GetEAa;*rmrd=LoadMd(eaa);SaveMd(eaa,LoadMd(eaa)+oldrmrd);}
+			break;
+		}
