@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2006  The DOSBox Team
+ *  Copyright (C) 2002-2007  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,7 +33,6 @@ static Bitu call_int16,call_irq1,call_irq6;
 
 /* Nice table from BOCHS i should feel bad for ripping this */
 #define none 0
-#define MAX_SCAN_CODE 0x58
 static struct {
   Bit16u normal;
   Bit16u shift;
@@ -131,7 +130,7 @@ static struct {
       { 0x8600, 0x8800, 0x8a00, 0x8c00 }  /* F12 */
       };
 
-static bool add_key_forced(Bit16u code) {
+bool BIOS_AddKeyToBuffer(Bit16u code) {
 	if (mem_readb(BIOS_KEYBOARD_FLAGS2)&8) return true;
 	Bit16u start,end,head,tail,ttail;
 	if (machine==MCH_PCJR) {
@@ -157,7 +156,7 @@ static bool add_key_forced(Bit16u code) {
 }
 
 static void add_key(Bit16u code) {
-	if (code!=0) add_key_forced(code);
+	if (code!=0) BIOS_AddKeyToBuffer(code);
 }
 
 static bool get_key(Bit16u &code) {
@@ -223,32 +222,14 @@ static bool check_key(Bit16u &code) {
 	*/
 
 
-
+/* the scancode is in reg_al */
 static Bitu IRQ1_Handler(void) {
-/* handling of the locks key is difficult as sdl only gives states for
- * numlock capslock. 
+/* handling of the locks key is difficult as sdl only gives
+ * states for numlock capslock. 
  */
-/* in reg_al is the scancode */
-
-	/* Read the code */
-	Bitu scancode; //,ascii,mod;
-#if 1
-	scancode=reg_al;    //IO_Read(0x60); moved out of handler
-
-
-#else
-	/* Old code capable of unicode keys. Dropped Readkey disabled in keyboard.cpp */   
-	KEYBOARD_ReadKey(scancode,ascii,mod);
-	LOG_MSG("Got code %X ascii %C mod %X",scancode,ascii,mod);
-#endif	
-	Bit16u old_ax=reg_ax;
-	reg_flags|=1;
-	reg_ah=0x4f;reg_al=scancode;
-	CALLBACK_RunRealInt(0x15);
-	reg_ax=old_ax;Bit8u flags1,flags2,flags3,leds;
-	if (!(reg_flags&1)) goto irq1_return;
-
+	Bitu scancode=reg_al;	/* Read the code */
 
+	Bit8u flags1,flags2,flags3,leds;
 	flags1=mem_readb(BIOS_KEYBOARD_FLAGS1);
 	flags2=mem_readb(BIOS_KEYBOARD_FLAGS2);
 	flags3=mem_readb(BIOS_KEYBOARD_FLAGS3);
@@ -258,6 +239,7 @@ static Bitu IRQ1_Handler(void) {
 #else
 	flags2&=~(0x40+0x20);//remove numlock/capslock pressed (hack for sdl only reporting states)
 #endif
+	if (DOS_LayoutKey(scancode,flags1,flags2,flags3)) return CBRET_NONE;
 	switch (scancode) {
 	/* First the hard ones  */
 	case 0xfa:	/* ack. Do nothing for now */
@@ -334,7 +316,7 @@ static Bitu IRQ1_Handler(void) {
 				mem_writeb(BIOS_KEYBOARD_FLAGS2,flags2|8);
 				IO_Write(0x20,0x20);
 				while (mem_readb(BIOS_KEYBOARD_FLAGS2)&8) CALLBACK_Idle();	// pause loop
-				reg_ip+=4;	// skip out 20,20
+				reg_ip+=5;	// skip out 20,20
 				return CBRET_NONE;
 			}
 		} else {
@@ -460,7 +442,6 @@ irq1_end:
 	mem_writeb(BIOS_KEYBOARD_FLAGS2,flags2);
 	mem_writeb(BIOS_KEYBOARD_FLAGS3,flags3);
 	mem_writeb(BIOS_KEYBOARD_LEDS,leds);
-irq1_return:
 /*	IO_Write(0x20,0x20); moved out of handler to be virtualizable */
 #if 0
 /* Signal the keyboard for next code */
@@ -472,22 +453,6 @@ irq1_return:
 	return CBRET_NONE;
 }
 
-static Bitu IRQ6_Handler(void) {
-	Bit8u scancode=IO_Read(0x60);
-	/* skip extended keys, all of them should map quite nicely
-	   onto corresponding non-extended keys */
-	if (scancode!=0xe0) {
-		Bit16u old_ax=reg_ax;
-		reg_al=scancode;
-		/* call the real keyboard IRQ now, with the scancode in AL */
-		CALLBACK_RunRealInt(0x09);
-		reg_ax=old_ax;
-	}
-
-	IO_Write(0x20,0x20);
-	return CBRET_NONE;
-}
-
 
 /* check whether key combination is enhanced or not,
    translate key if necessary */
@@ -519,32 +484,29 @@ static Bitu INT16_Handler(void) {
 	Bit16u temp=0;
 	switch (reg_ah) {
 	case 0x00: /* GET KEYSTROKE */
-		for (;;) {
-			if (get_key(temp)) {
-				if (!IsEnhancedKey(temp)) {
-					/* normal key, exit scanning for keys */
-					break;
-				}
-			}
-			CALLBACK_Idle();
+		if ((get_key(temp)) && (!IsEnhancedKey(temp))) {
+			/* normal key found, return translated key in ax */
+			reg_ax=temp;
+		} else {
+			/* enter small idle loop to allow for irqs to happen */
+			reg_ip+=1;
 		}
-		/* normal key found, return translated key in ax */
-		reg_ax=temp;
 		break;
 	case 0x10: /* GET KEYSTROKE (enhanced keyboards only) */
-		for (;;) {
-			if (get_key(temp)) {
-				if (((temp&0xff)==0xf0) && (temp>>8)) {
-					/* special enhanced key, clear low part before returning key */
-					temp&=0xff00;
-				}
-				break;
+		if (get_key(temp)) {
+			if (((temp&0xff)==0xf0) && (temp>>8)) {
+				/* special enhanced key, clear low part before returning key */
+				temp&=0xff00;
 			}
-			CALLBACK_Idle();
+			reg_ax=temp;
+		} else {
+			/* enter small idle loop to allow for irqs to happen */
+			reg_ip+=1;
 		}
-		reg_ax=temp;
 		break;
 	case 0x01: /* CHECK FOR KEYSTROKE */
+		// enable interrupt-flag after IRET of this int16
+		mem_writew(SegPhys(ss)+reg_sp+4,(mem_readw(SegPhys(ss)+reg_sp+4) | FLAG_IF));
 		for (;;) {
 			if (check_key(temp)) {
 				if (!IsEnhancedKey(temp)) {
@@ -561,7 +523,7 @@ static Bitu INT16_Handler(void) {
 				CALLBACK_SZF(true);
 				break;
 			}
-			CALLBACK_Idle();
+//			CALLBACK_Idle();
 		}
 		break;
 	case 0x11: /* CHECK FOR KEYSTROKE (enhanced keyboards only) */
@@ -591,7 +553,7 @@ static Bitu INT16_Handler(void) {
 		}
 		break;
 	case 0x05:	/* STORE KEYSTROKE IN KEYBOARD BUFFER */
-		if (add_key_forced(reg_cx)) reg_al=0;
+		if (BIOS_AddKeyToBuffer(reg_cx)) reg_al=0;
 		else reg_al=1;
 		break;
 	case 0x12: /* GET EXTENDED SHIFT STATES */
@@ -630,37 +592,51 @@ static void InitBiosSegment(void) {
 	mem_writeb(BIOS_KEYBOARD_FLAGS3,16); /* Enhanced keyboard installed */	
 	mem_writeb(BIOS_KEYBOARD_TOKEN,0);
 	mem_writeb(BIOS_KEYBOARD_LEDS,leds);
-
 }
 
 void BIOS_SetupKeyboard(void) {
 	/* Init the variables */
 	InitBiosSegment();
-	/* Allocate a callback for int 0x16 and for standard IRQ 1 handler */
+
+	/* Allocate/setup a callback for int 0x16 and for standard IRQ 1 handler */
 	call_int16=CALLBACK_Allocate();	
-	call_irq1=CALLBACK_Allocate();	
-	CALLBACK_Setup(call_int16,&INT16_Handler,CB_IRET_STI,"keyboard");
+	CALLBACK_Setup(call_int16,&INT16_Handler,CB_INT16,"keyboard");
 	RealSetVec(0x16,CALLBACK_RealPointer(call_int16));
-	CALLBACK_Setup(call_irq1,&IRQ1_Handler,CB_IRET,"keyboard irq");
+
+	call_irq1=CALLBACK_Allocate();	
+	CALLBACK_Setup(call_irq1,&IRQ1_Handler,CB_IRQ1,"keyboard irq");
 	RealSetVec(0x9,CALLBACK_RealPointer(call_irq1));
+	// pseudocode for CB_IRQ1:
+	//	push ax
+	//	in al, 0x60
+	//	mov ah, 0x4f
+	//	stc
+	//	int 15
+	//	jc skip
+	//	callback IRQ1_Handler
+	//	label skip:
+	//	cli
+	//	mov al, 0x20
+	//	out 0x20, al
+	//	pop ax
+	//	iret
+
 	if (machine==MCH_PCJR) {
 		call_irq6=CALLBACK_Allocate();
-		CALLBACK_Setup(call_irq6,&IRQ6_Handler,CB_IRET,"PCJr kb irq");
+		CALLBACK_Setup(call_irq6,NULL,CB_IRQ6_PCJR,"PCJr kb irq");
 		RealSetVec(0x0e,CALLBACK_RealPointer(call_irq6));
+		// pseudocode for CB_IRQ6_PCJR:
+		//	push ax
+		//	in al, 0x60
+		//	cmp al, 0xe0
+		//	je skip
+		//	int 0x09
+		//	label skip:
+		//	cli
+		//	mov al, 0x20
+		//	out 0x20, al
+		//	pop ax
+		//	iret
 	}
-
-	/* bring the all port operations outside the callback */
-	phys_writeb(CB_BASE+(call_irq1<<4)+0x00,(Bit8u)0x50);		// push ax
-	phys_writeb(CB_BASE+(call_irq1<<4)+0x01,(Bit8u)0xe4);		// in al, 0x60
-	phys_writeb(CB_BASE+(call_irq1<<4)+0x02,(Bit8u)0x60);
-	phys_writeb(CB_BASE+(call_irq1<<4)+0x03,(Bit8u)0xFE);		//GRP 4
-	phys_writeb(CB_BASE+(call_irq1<<4)+0x04,(Bit8u)0x38);		//Extra Callback instruction
-	phys_writew(CB_BASE+(call_irq1<<4)+0x05,call_irq1);		//The immediate word
-	phys_writeb(CB_BASE+(call_irq1<<4)+0x07,(Bit8u)0xb0);		// mov al, 0x20
-	phys_writeb(CB_BASE+(call_irq1<<4)+0x08,(Bit8u)0x20);
-	phys_writeb(CB_BASE+(call_irq1<<4)+0x09,(Bit8u)0xe6);		// out 0x20, al
-	phys_writeb(CB_BASE+(call_irq1<<4)+0x0a,(Bit8u)0x20);
-	phys_writeb(CB_BASE+(call_irq1<<4)+0x0b,(Bit8u)0x58);		// pop ax
-	phys_writeb(CB_BASE+(call_irq1<<4)+0x0c,(Bit8u)0xcf);		// iret
 }
 
